@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
  *
- * Copyright (C) 2006 Richard Hughes <richard@hughsie.com>
+ * Copyright (C) 2006-2007 Richard Hughes <richard@hughsie.com>
  *
  * Licensed under the GNU General Public License Version 2
  *
@@ -26,14 +26,17 @@
 #include <glib/gi18n.h>
 
 #include <X11/X.h>
-#include <glib/gi18n.h>
 #include <gdk/gdkx.h>
 #include <gtk/gtk.h>
 
+#include <libhal-gmanager.h>
+#include <libhal-gdevice.h>
+#include <libhal-gdevicestore.h>
+
 #include "gpm-common.h"
 #include "gpm-button.h"
 #include "gpm-debug.h"
-#include "gpm-hal.h"
+#include "gpm-sound.h"
 #include "gpm-marshal.h"
 
 static void     gpm_button_class_init (GpmButtonClass *klass);
@@ -46,9 +49,11 @@ struct GpmButtonPrivate
 {
 	GdkScreen		*screen;
 	GdkWindow		*window;
+	GpmSound		*sound;
 	GHashTable		*hash_to_hal;
 	gboolean		 lid_is_closed;
-	GpmHal			*hal; /* remove when iput events is in the kernel */
+	HalGManager		*hal_manager; /* remove when input events is in the kernel */
+	HalGDevicestore		*hal_devicestore;
 };
 
 enum {
@@ -56,7 +61,8 @@ enum {
 	LAST_SIGNAL
 };
 
-static guint	     signals [LAST_SIGNAL] = { 0, };
+static guint signals [LAST_SIGNAL] = { 0, };
+static gpointer gpm_button_object = NULL;
 
 G_DEFINE_TYPE (GpmButton, gpm_button, G_TYPE_OBJECT)
 
@@ -71,26 +77,29 @@ gpm_button_filter_x_events (GdkXEvent *xevent,
 	gchar *hashkey;
 	gchar *key;
 
-	keycode = xev->xkey.keycode;
-	state = xev->xkey.state;
-	
 	if (xev->type != KeyPress) {
-		hashkey = g_strdup_printf ("key_%x_%x", state, keycode);
-
-		/* is the key string already in our DB? */
-		key = g_hash_table_lookup (button->priv->hash_to_hal, hashkey);
-		if (key == NULL) {
-			gpm_warning ("Key '%s' not found in hash!", hashkey);
-		} else {
-			gpm_debug ("Key '%s' mapped to HAL key %s", hashkey, key);
-			g_signal_emit (button, signals [BUTTON_PRESSED], 0, key);
-		}
-
-		g_free (hashkey);
-		return GDK_FILTER_REMOVE;
+		return GDK_FILTER_CONTINUE;
 	}
 
-	return GDK_FILTER_CONTINUE;
+	keycode = xev->xkey.keycode;
+	state = xev->xkey.state;
+
+	hashkey = g_strdup_printf ("key_%x_%x", state, keycode);
+
+	/* is the key string already in our DB? */
+	key = g_hash_table_lookup (button->priv->hash_to_hal, hashkey);
+	if (key == NULL) {
+		gpm_debug ("Key '%s' not found in hash", hashkey);
+		/* pass normal keypresses on, which might help with accessibility access */
+		g_free (hashkey);
+		return GDK_FILTER_CONTINUE;
+	}
+
+	gpm_debug ("Key '%s' mapped to HAL key %s", hashkey, key);
+	g_signal_emit (button, signals [BUTTON_PRESSED], 0, key);
+
+	g_free (hashkey);
+	return GDK_FILTER_REMOVE;
 }
 
 /**
@@ -183,8 +192,8 @@ gpm_button_grab_keystring (GpmButton   *button,
  **/
 static gboolean
 gpm_button_button_key (GpmButton   *button,
-			   const gchar *keystr,
-			   const gchar *hal_key)
+		       const gchar *keystr,
+		       const gchar *hal_key)
 {
 	char *key = NULL;
 	gboolean ret;
@@ -201,7 +210,7 @@ gpm_button_button_key (GpmButton   *button,
 	if (ret == FALSE) {
 		gpm_warning ("Failed to grab %s", keystr);
 		return FALSE;
-	}	
+	}
 
 	/* add to hash table */
 	g_hash_table_insert (button->priv->hash_to_hal, key, (gpointer) hal_key);
@@ -233,11 +242,13 @@ gpm_button_class_init (GpmButtonClass *klass)
 }
 
 /**
- * button_is_lid_closed:
+ * gpm_button_is_lid_closed:
  **/
 gboolean
-button_is_lid_closed (GpmButton *button)
+gpm_button_is_lid_closed (GpmButton *button)
 {
+	g_return_val_if_fail (button != NULL, FALSE);
+	g_return_val_if_fail (GPM_IS_BUTTON (button), FALSE);
 	return button->priv->lid_is_closed;
 }
 
@@ -258,23 +269,26 @@ button_is_lid_closed (GpmButton *button)
  *      button.type=""
  */
 static void
-emit_button_pressed (GpmButton *button,
-		     const gchar   *udi,
-		     const gchar   *details)
+emit_button_pressed (GpmButton   *button,
+		     HalGDevice  *device,
+		     const gchar *details)
 {
 	gchar *type = NULL;
 	gboolean state;
 	const char *atype;
 
-	g_return_if_fail (udi != NULL);
+	g_return_if_fail (device != NULL);
 	g_return_if_fail (details != NULL);
 
 	if (strcmp (details, "") == 0) {
 		/* no details about the event, so we get more info
 		   for type 1 buttons */
-		gpm_hal_device_get_string (button->priv->hal, udi, "button.type", &type);
+		hal_gdevice_get_string (device, "button.type", &type, NULL);
 		/* hal may no longer be there */
-		g_return_if_fail (type != NULL);
+		if (type == NULL) {
+			gpm_warning ("cannot get button type for %s", hal_gdevice_get_udi (device));
+			return;
+		}
 	} else {
 		type = g_strdup (details);
 	}
@@ -284,7 +298,7 @@ emit_button_pressed (GpmButton *button,
 	state = TRUE;
 	/* we need to get the button state for lid buttons */
 	if (strcmp (type, "lid") == 0) {
-		gpm_hal_device_get_bool (button->priv->hal, udi, "button.state.value", &state);
+		hal_gdevice_get_bool (device, "button.state.value", &state, NULL);
 	}
 
 	/* abstact away that HAL has an extra parameter */
@@ -297,17 +311,19 @@ emit_button_pressed (GpmButton *button,
 	/* filter out duplicate lid events */
 	if (strcmp (atype, GPM_BUTTON_LID_CLOSED) == 0) {
 		if (button->priv->lid_is_closed == TRUE) {
-			g_debug ("ignoring duplicate lid event");
+			gpm_debug ("ignoring duplicate lid event");
 			return;
 		}
 		button->priv->lid_is_closed = TRUE;
+		gpm_sound_event (button->priv->sound, GPM_SOUND_LID_DOWN);
 	}
 	if (strcmp (atype, GPM_BUTTON_LID_OPEN) == 0) {
 		if (button->priv->lid_is_closed == FALSE) {
-			g_debug ("ignoring duplicate lid event");
+			gpm_debug ("ignoring duplicate lid event");
 			return;
 		}
 		button->priv->lid_is_closed = FALSE;
+		gpm_sound_event (button->priv->sound, GPM_SOUND_LID_UP);
 	}
 
 	/* the names changed in 0.5.8 */
@@ -336,26 +352,25 @@ emit_button_pressed (GpmButton *button,
  * changed, and we have we have subscribed to changes for that device.
  */
 static void
-hal_device_property_modified_cb (GpmHal        *hal,
-				 const gchar   *udi,
+hal_device_property_modified_cb (HalGDevice    *device,
 				 const gchar   *key,
 				 gboolean       is_added,
 				 gboolean       is_removed,
 				 gboolean       finally,
-				 GpmButton *button)
+				 GpmButton     *button)
 {
-	gpm_debug ("udi=%s, key=%s, added=%i, removed=%i, finally=%i",
-		   udi, key, is_added, is_removed, finally);
+	gpm_debug ("key=%s, added=%i, removed=%i, finally=%i",
+		   key, is_added, is_removed, finally);
 
 	/* do not process keys that have been removed */
-	if (is_removed) {
+	if (is_removed == TRUE) {
 		return;
 	}
 
 	/* only match button* values */
 	if (strncmp (key, "button", 6) == 0) {
-		gpm_debug ("state of a button has changed : %s, %s", udi, key);
-		emit_button_pressed (button, udi, "");
+		gpm_debug ("state of a button has changed : %s, %s", hal_gdevice_get_udi (device), key);
+		emit_button_pressed (button, device, "");
 	}
 }
 
@@ -370,16 +385,15 @@ hal_device_property_modified_cb (GpmHal        *hal,
  * changed, and we have we have subscribed to changes for that device.
  */
 static void
-hal_device_condition_cb (GpmHal        *hal,
-			 const gchar   *udi,
+hal_device_condition_cb (HalGDevice    *device,
 			 const gchar   *condition,
 			 const gchar   *details,
-			 GpmButton *button)
+			 GpmButton     *button)
 {
-	gpm_debug ("udi=%s, condition=%s, details=%s", udi, condition, details);
+	gpm_debug ("condition=%s, details=%s", condition, details);
 
 	if (strcmp (condition, "ButtonPressed") == 0) {
-		emit_button_pressed (button, udi, details);
+		emit_button_pressed (button, device, details);
 	}
 }
 
@@ -392,8 +406,80 @@ static void
 watch_add_button (GpmButton *button,
 		  const gchar   *udi)
 {
-	gpm_hal_device_watch_condition (button->priv->hal, udi, FALSE);
-	gpm_hal_device_watch_propery_modified (button->priv->hal, udi, FALSE);
+	HalGDevice *device;
+
+	device = hal_gdevice_new ();
+	hal_gdevice_set_udi (device, udi);
+	hal_gdevice_watch_condition (device);
+	hal_gdevice_watch_property_modified (device);
+
+	g_signal_connect (device, "property-modified",
+			  G_CALLBACK (hal_device_property_modified_cb), button);
+	g_signal_connect (device, "device-condition",
+			  G_CALLBACK (hal_device_condition_cb), button);
+
+	/* when added to the devicestore, devices are automatically unref'ed */
+	hal_gdevicestore_insert (button->priv->hal_devicestore, device);
+}
+
+/**
+ * coldplug_buttons:
+ **/
+static void
+coldplug_buttons (GpmButton *button)
+{
+	int i;
+	char **device_names = NULL;
+	gboolean ret;
+	GError *error;
+
+	/* devices of type button */
+	error = NULL;
+	ret = hal_gmanager_find_capability (button->priv->hal_manager, "button", &device_names, &error);
+	if (ret == FALSE) {
+		gpm_warning ("Couldn't obtain list of buttons: %s", error->message);
+		g_error_free (error);
+		return;
+	}
+	if (device_names[0] != NULL) {
+		/* we have found buttons */
+		for (i = 0; device_names[i]; i++) {
+			watch_add_button (button, device_names[i]);
+			gpm_debug ("Watching %s", device_names[i]);
+		}
+	} else {
+		gpm_debug ("Couldn't obtain list of buttons");
+	}
+
+	hal_gmanager_free_capability (device_names);
+}
+
+/**
+ * hal_daemon_start_cb:
+ **/
+static void
+hal_daemon_start_cb (HalGManager *hal_manager,
+		     GpmButton   *button)
+{
+	/* get new devices, hal has come back up */
+	if (button->priv->hal_devicestore == NULL) {
+		button->priv->hal_devicestore = hal_gdevicestore_new ();
+		coldplug_buttons (button);
+	}
+}
+
+/**
+ * hal_daemon_stop_cb:
+ **/
+static void
+hal_daemon_stop_cb (HalGManager *hal_manager,
+		    GpmButton   *button)
+{
+	/* clear devices, HAL is going down */
+	if (button->priv->hal_devicestore != NULL) {
+		g_object_unref (button->priv->hal_devicestore);
+		button->priv->hal_devicestore = NULL;
+	}
 }
 
 /**
@@ -404,8 +490,6 @@ static void
 gpm_button_init (GpmButton *button)
 {
 	gboolean have_xevents = FALSE;
-	int    i;
-	char **device_names = NULL;
 
 	button->priv = GPM_BUTTON_GET_PRIVATE (button);
 
@@ -415,6 +499,7 @@ gpm_button_init (GpmButton *button)
 	button->priv->hash_to_hal = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, NULL);
 
 	button->priv->lid_is_closed = FALSE;
+	button->priv->sound = gpm_sound_new ();
 
 #ifdef HAVE_XEVENTS
 	have_xevents = TRUE;
@@ -441,25 +526,15 @@ gpm_button_init (GpmButton *button)
 	}
 
 	/* remove when button support is out of HAL */
-	button->priv->hal = gpm_hal_new ();
+	button->priv->hal_manager = hal_gmanager_new ();
+	g_signal_connect (button->priv->hal_manager, "daemon-start",
+			  G_CALLBACK (hal_daemon_start_cb), button);
+	g_signal_connect (button->priv->hal_manager, "daemon-stop",
+			  G_CALLBACK (hal_daemon_stop_cb), button);
 
-	/* devices of type button */
-	gpm_hal_device_find_capability (button->priv->hal, "button", &device_names);
-	if (! device_names) {
-		gpm_debug ("Couldn't obtain list of buttons");
-		return;
-	}
+	button->priv->hal_devicestore = hal_gdevicestore_new ();
 
-	for (i = 0; device_names[i]; i++) {
-		watch_add_button (button, device_names [i]);
-	}
-
-	gpm_hal_free_capability (button->priv->hal, device_names);
-
-	g_signal_connect (button->priv->hal, "property-modified",
-			  G_CALLBACK (hal_device_property_modified_cb), button);
-	g_signal_connect (button->priv->hal, "device-condition",
-			  G_CALLBACK (hal_device_condition_cb), button);
+	coldplug_buttons (button);
 }
 
 /**
@@ -476,11 +551,13 @@ gpm_button_finalize (GObject *object)
 	button = GPM_BUTTON (object);
 	button->priv = GPM_BUTTON_GET_PRIVATE (button);
 
-	if (button->priv->hal != NULL) {
-		g_object_unref (button->priv->hal);
-	}
+	g_object_unref (button->priv->hal_manager);
+	g_object_unref (button->priv->hal_devicestore);
+	g_object_unref (button->priv->sound);
 
 	g_hash_table_unref (button->priv->hash_to_hal);
+
+	G_OBJECT_CLASS (gpm_button_parent_class)->finalize (object);
 }
 
 /**
@@ -490,11 +567,11 @@ gpm_button_finalize (GObject *object)
 GpmButton *
 gpm_button_new (void)
 {
-	static GpmButton *button = NULL;
-	if (button != NULL) {
-		g_object_ref (button);
+	if (gpm_button_object != NULL) {
+		g_object_ref (gpm_button_object);
 	} else {
-		button = g_object_new (GPM_TYPE_BUTTON, NULL);
+		gpm_button_object = g_object_new (GPM_TYPE_BUTTON, NULL);
+		g_object_add_weak_pointer (gpm_button_object, &gpm_button_object);
 	}
-	return GPM_BUTTON (button);
+	return GPM_BUTTON (gpm_button_object);
 }
