@@ -1187,6 +1187,10 @@ phone_device_removed_cb (GpmPhone     *phone,
 	GpmCellUnit *unit;
 
 	unit = &(cell_array->priv->unit);
+	if (unit == NULL) {
+		gpm_warning ("unit was NULL!");
+		return;
+	}
 
 	/* ignore non-phones */
 	if (unit->kind != GPM_CELL_UNIT_KIND_PHONE) {
@@ -1195,6 +1199,10 @@ phone_device_removed_cb (GpmPhone     *phone,
 
 	/* we unref as the device has gone away (only support one phone) */
 	cell = (GpmCell *) g_ptr_array_index (cell_array->priv->array, 0);
+	if (cell == NULL) {
+		gpm_warning ("cell was NULL - gnome-phone-manager is playing with us!");
+		return;
+	}
 	g_object_unref (cell);
 
 	/* remove from the devicestore */
@@ -1480,17 +1488,21 @@ gpm_st_cell_array (GpmSelfTest *test)
 
 	/************************************************************/
 	gpm_st_title (test, "make sure we got a single recall notice");
+	if (recall_count2 == 0) {
+		gpm_st_warning (test, "did not get recall - install fdi!");
+		goto out;
+	}
 	if (recall_count2 == 1) {
 		gpm_st_success (test, "got recall");
 	} else {
-		gpm_st_failed (test, "did not get recall (install fdi?)");
+		gpm_st_failed (test, "got duplicate recall");
 	}
 
 	/************************************************************/
 	gpm_st_title (test, "make sure we got at least 1 cell");
 	count = gpm_cell_array_get_num_cells (cell_array);
 	if (count >= 1) {
-	gpm_st_success (test, "got %d cell(s)", count);
+		gpm_st_success (test, "got %d cell(s)", count);
 	} else {
 		gpm_st_failed (test, "got %i cells", count);
 	}
@@ -1514,6 +1526,7 @@ gpm_st_cell_array (GpmSelfTest *test)
 		gpm_st_failed (test, "could not get correct cell");
 	}
 
+out:
 	g_object_unref (cell_array);
 
 	gpm_st_end (test);
