@@ -121,6 +121,10 @@ gpm_cell_refresh_hal_all (GpmCell *cell)
 						&unit->is_charging, NULL);
 			hal_gdevice_get_bool (device, "battery.rechargeable.is_discharging",
 						&unit->is_discharging, NULL);
+		} else {
+			/* HAL isn't being helpful here... */
+			unit->is_discharging = TRUE;
+			unit->is_charging = FALSE;
 		}
 	} else {
 		/* devices cannot charge, well, at least not while being used */
@@ -166,7 +170,12 @@ gpm_cell_refresh_hal_all (GpmCell *cell)
 						   "%i to 100", unit->capacity);
 					unit->capacity = 100;
 				}
-				if (unit->capacity > 0 && unit->capacity < 50) {
+				if (unit->capacity < 0) {
+					gpm_debug ("rounding up capactity from "
+						   "%i to 0", unit->capacity);
+					unit->capacity = 0;
+				}
+				if (unit->capacity < 50) {
 					gpm_warning ("battery has a low capacity");
 					gpm_debug ("** EMIT: low-capacity");
 					g_signal_emit (cell, signals [LOW_CAPACITY], 0, unit->capacity);
@@ -439,18 +448,27 @@ gpm_cell_get_icon (GpmCell *cell)
 gchar *
 gpm_cell_get_id (GpmCell *cell)
 {
+	gchar *id = NULL;
+
 	g_return_val_if_fail (cell != NULL, NULL);
 	g_return_val_if_fail (GPM_IS_CELL (cell), NULL);
 
 	/* try to use these in order of uniqueness */
 	if (cell->priv->serial != NULL && strlen (cell->priv->serial) > 2) {
-		return g_strdup (cell->priv->serial);
+		/* serial number */
+		id = g_strdup (cell->priv->serial);
+	} else if (cell->priv->model != NULL && strlen (cell->priv->model) > 2) {
+		/* model number */
+		id = g_strdup (cell->priv->model);
+	} else {
+		/* just return something generic */
+		id = g_strdup ("generic_id");
 	}
-	if (cell->priv->model != NULL && strlen (cell->priv->model) > 2) {
-		return g_strdup (cell->priv->model);
-	}
-	/* if we have no info, just return something generic */
-	return g_strdup ("generic_id");
+
+	/* the id may have invalid chars that need to be replaced */
+	g_strdelimit (id, "\\\t\"' /", '_');
+
+	return id;
 }
 
 /**
